
:- use_module( library(options) ).
:- use_module( library(real) ).
:- <- library("ggplot2").

:- requires( head/2 ).
:- requires( positions/3 ).
:- requires( kv_decompose/3 ).
:- requires( kv_ks/2 ).
% :- requires( gg_hue_colour_strings/2 ).

gg_bar_plot_defaults( ArgS, Defs ) :-
     ( is_list(ArgS) -> Args=ArgS; Args=[ArgS] ),
	DefFlip = true,
	( memberchk(flip(Flip),Args) -> true; Flip = DefFlip ),
	( Flip == true -> Rev = true; Rev = false ),
	Defs = [ debug(false),
	         df_rvar(gbp_df), df_rvar_rmv(true), 
		    fill_colours(false),
		    geom_bar(true), geom_bar_draw_colour(white),
		    geom_bar_position(dodge),
		    gg_terms([]),
	         flip(DefFlip), 
		    keep_order(false),
		    labels('','',''),
	         legend_labels([]),
	         legend_reverse(Rev),
		    legend_title([]),
		    extra_legend(false),
		    extra_legend_position(12,182,0,4)
	       ].

%% gg_bar_plot( +Pairs, +Opts ).
%
% Plot a bar plot of population Pairs (Name-Populations).
%  
% Opts is a combination of controlling the predicate as per normal
% Prolog convention, and term structures that tranlate to '+' ggplot2 terms.
%
% Opts
% * bar_draw_colour(Bdc=white) draw colour for bars
% * debug(Dbg=false)           _true_ turns debugging on
% * df_rvar(Df=gbp_df)         data frame R variable
% * df_rvar_rmv(DfR=true)      remove Df after call
% * extra_legend(XtLeg=false)    
%   adds extra legend
% * extra_legend_position(Xn,Xx,Yn,Yx)
%   position for extra legend
% * fill_colours(Clrs=true)    _false_ removes scale_fill_manual() term, _true_ use def. colours, else give list of colours
% * flip(Flip=true)            flip coordinates
% * geom_bar(Gb=true)
%  _false_ removes removes geom_bar() term
% * geom_bar_draw_colour(Gc=white)
%   colour for bar outlines
% * geom_bar_position(Gp=dodge)
%   or _stack_; positions bars
% * gg_terms([])
%   arbitrary terms to add to the plot
% * labels(X,Y,Main)
%   X='',Y='',Main='', if Flip==true then you should also swap X with Y here
% * legend_reverse(Lrv=true)
%   reverse legend order (Lrv=false when Flip=false)
% * legend_title(Lt)
%   legend title ([] for default ggplot2() one). only works for Clrs \== false
% * legend_labels(Ll=[])
%   labels for legend
% * keep_order(Kord=false)     display x axis according to given order (default uses lexicograpical sort)
%   
%==
% ?- Pairs = [a-[1,2,3],b-[2,4,6]], gg_bar_plot( Pairs, true ).
%
% ?- FClrs = ["gold1", "#E31A1C", "blue1"].
%    BoldTitle = theme(plot.title(element_text(face(+"bold")))),
%    Pairs = [a-[1,2,3],b-[2,4,6]],
%    gg_bar_plot( Pairs, [debug(true), geom_bar_draw_colour(black), labels(x,y,main), fill_colours(FClrs), gg_terms(BoldTitle),legend_title(leeg)] ).
%==
%
% @author nicos angelopoulos
% @version  0.1 2014/10/21
% @version  0.2 2016/01/23
%
gg_bar_plot( Pairs, Args ) :-
	options_append( gg_bar_plot, Args, Opts, process(debug) ),
	gg_bar_plot_base( Pairs, Df, Len, GGbase, Opts ),
	options( flip(Flip), Opts ),
	gg_bar_plot_flip_term( Flip, GGbase, GGflip ),
	options( legend_reverse(Lrv), Opts ),
	gg_bar_plot_leg_reverse( Lrv, GGflip, GGlrev ),
	GBs =[geom_bar_draw_colour(Gbc),geom_bar(Gbb),geom_bar_position(Gbp)],  
	options( GBs, Opts ),
	gg_bar_plot_geom_bar( Gbb, Gbp, Gbc, GGlrev, GGgb ),
	options( fill_colours(Fclrs), Opts ),
	options( legend_title(Ltitle), Opts ),
	options( legend_labels(LLbls), Opts ),
	debug( gg_bar_plot, 'Legend labels: ~w', [LLbls] ),
	gg_bar_plot_fill_colours( Fclrs, Ltitle, Len, LLbls, GGgb, GGfill ),
	options( labels(Xlbl,Ylbl,Mlbl), Opts ),
	GGlbl = ( GGfill + labs( x=+Xlbl, y=+Ylbl, title=+Mlbl ) ),
	% GG = ( GGlbl + theme(plot.title=element_text(face=+bold) ) ),
	options( gg_terms(PlTermS), Opts ),
	en_list( PlTermS, PlTerms ),
	gg_bar_plot_opts( PlTerms, GGlbl, GG ),
	debug( gg_bar_plot, 'GG bar plot term: ~w', [GG] ),
	% <- print( GG + theme( panel.background='element_blank()', panel.grid='element_blank()', axis.text.x = element_text(colour="black"), axis.text.y = element_text(colour="black") ) ),
	options( [extra_legend(XtraLeg),extra_legend_position(Xn,Xx,Yn,Yx)], Opts ),  % experimental
	gg_bar_plot_extra_legend_plots( XtraLeg, Xn, Xx, Yn, Yx, GG ),
	options( df_rvar_rmv(Rmv), Opts ),
	gg_bar_plot_rvar_remove( Rmv, Df ).

gg_bar_plot_extra_legend_plots( false, _, _, _, _, GG ) :-
	<- print( GG ).
gg_bar_plot_extra_legend_plots( [H|T], Xn, Xx, Yn, Yx, GG ) :-
	kv_decompose( [H|T], Labels, Clrs ),
	<- library( "gridExtra" ),
	<- library( "gtable" ),
	length( Clrs, Len ),
	CClrs =.. [c|Clrs],
	tmp_df <- data.frame( x=1:Len, y=1:Len, 'X.label'=factor(Labels) ),
	tmp_plot1 <- ggplot( data = tmp_df, aes(x=x,y=y))
	             + geom_point( aes(colour='X.label') )
			   + scale_colour_manual(values=CClrs)
			   + theme(legend.position="right"),
	tmp_leg1 <- gtable_filter(ggplot_gtable(ggplot_build(tmp_plot1)), "guide-box"),
	% fixme make the following x+ys in to options
	tmp_plot_new <- GG + annotation_custom( grob=tmp_leg1, xmin=Xn, xmax=Xx, ymin=Yn, ymax=Yx ),
	tmp_plot_new <- arrangeGrob( tmp_leg1, tmp_plot_new, 
	                  heights = unit.c( tmp_leg1$height,unit(1,"npc") - tmp_leg1$height), ncol=1 ),
	<- grid.newpage(),
	<- grid.draw(tmp_plot_new).

gg_bar_plot_base( Pairs, Df, Len, GGbase, Opts ) :-
	head( Pairs, _-FstPops ),
	length( FstPops, Len ),
	positions( FstPops, Dtype, Poss ),
	maplist( gg_bar_plot_kvs(Dtype,Poss), Pairs, DataKVs ),
	flatten( DataKVs, KVs ),
	kv_decompose( KVs, ClustPosPs, Pops ),
	kv_decompose( ClustPosPs, Clusts, Posits ),
	options( df_rvar(Df), Opts ),
	Df <- 'data.frame'( pop=Pops, clu=Clusts, pos=Posits  ),
	options( keep_order(Kord), Opts ),
	gg_bar_plot_base_x_order( Kord, Pairs, Df ),
	flatten( DataKVs, KVs ),
	kv_decompose( KVs, ClustPosPs, Pops ),
	kv_decompose( ClustPosPs, Clusts, Posits ),
	% GGbase = 	ggplot(Df, drop='T', aes(x=as.factor(clu), y=pop, fill=as.factor(pos), levels=Ks)).
	% % GGbase = 	ggplot(Df, drop='T', aes(x=as.factor(clu), y=pop, fill=as.factor(pos))).
	GGbase = 	ggplot(Df, aes(x=as.factor(clu), y=pop, fill=as.factor(pos))).

gg_bar_plot_base_x_order( false, _Pairs, _Df ).
gg_bar_plot_base_x_order( true, Pairs, Df ) :-
	kv_ks( Pairs, Ks ),
	Df$clu <- factor( Df$clu, levels=Ks ).

gg_bar_plot_kvs( list, _Poss, Clust-Pops, DataKVs ) :-
	findall( Clust-Pos-Pop, nth1(Pos,Pops,Pop), DataKVs ).
gg_bar_plot_kvs( compound, _Poss, Clust-Pops, DataKVs ) :-
	!, 
	functor( Pops, _, Arity ),
	findall( Clust-Pos-Pop, (between(1,Arity,Pos),arg(Pos,Pops,Pop)), DataKVs ).

gg_bar_plot_leg_reverse( true, GG, (GG + guides(fill = guide_legend(reverse='TRUE'))) ).
gg_bar_plot_leg_reverse( false, GG, GG ).

gg_bar_plot_flip_term( true, GG, (GG + 'coord_flip()') ).
gg_bar_plot_flip_term( false, GG, GG ).

gg_bar_plot_geom_bar( fill, _Pos, Clr, GG, (GG+Geom) ) :-
	% % Geom = geom_bar( position="fill", drop='F', stat=+identity, color=+Clr ).
	Geom = geom_bar( position="fill", stat=+identity, color=+Clr ).
gg_bar_plot_geom_bar( true, Pos, Clr, GG, (GG+Geom) ) :-
	% fixme: Clr = false, then remove +Clr
	% % Geom = geom_bar( position=+Pos, drop='F', stat=+identity, color=+Clr ).
	Geom = geom_bar( position=+Pos, stat=+identity, color=+Clr ).
	% Geom = geom_bar( position=+Pos, stat=+identity ).
gg_bar_plot_geom_bar( false, _Pos, _Clr, GG, GG ).

% gg_bar_plot_fill_colours( true, GG, GG ).
% gg_bar_plot_fill_colours( true, Title, Len, GG, (GG + scale_fill_manual(+Title,values=Vals)) ) :-
gg_bar_plot_fill_colours( CTerm, Title, _Len, Lbls, GG, (GG + scale_fill_manual(+Title,values=Vals,labels=FullLbls)) ) :-
	compound( CTerm ),
	functor( CTerm, c, Arity ),
	!,
	CTerm = Vals,
	( Lbls == [] -> numlist( 1, Arity, FullLblsL ); Lbls = FullLblsL ),
	maplist( atom_string, FullLblsL, FullLblsS ),
	FullLbls =.. [c|FullLblsS].
% gg_bar_plot_fill_colours( true, Title, _Len, Lbls, GG, (GG + scale_fill_discrete(+Title,positions=Lbls)) ).
% gg_bar_plot_fill_colours( true, Title, Len, InLbls, GG, (GG + scale_fill_discrete(+Title,values=Clrs,labels=Lbls)) ) :-
	% gg_hue_colour_strings( 30, Clrs ),
gg_bar_plot_fill_colours( true, Title, _Len, _InLbls, GG, (GG + scale_fill_discrete(+Title)) ).
gg_bar_plot_fill_colours( false, _Title, _Len, _Lbls, GG, GG ).
gg_bar_plot_fill_colours( [H|T], Title, _Len, Lbls, GG, (GG + scale_fill_manual(+Title,values=Vals,labels=FullLbls)) ) :-
	Vals =.. [c,H|T],
	( Lbls == [] -> length( [H|T], Len ), numlist( 1, Len, FullLblsL ); Lbls = FullLblsL ),
	maplist( atom_string, FullLblsL, FullLblsS ),
	FullLbls =.. [c|FullLblsS].

gg_bar_plot_rvar_remove( false, _Df ).
gg_bar_plot_rvar_remove( true, Df ) :-
	<- remove( Df ).

gg_bar_plot_opts( [], GG, GG ).
gg_bar_plot_opts( [H|T], GGbase, GGfull ) :-
	gg_bar_plot_opt( H, Hgg ),
	gg_bar_plot_opts( T, (GGbase + Hgg), GGfull ).

gg_bar_plot_opt( H, Hgg ) :-
	compound( H, Name, Args ),
	!,
	maplist( gg_bar_plot_gg_arg, Args, GGArgs ),
	compound( Hgg, Name, GGArgs ).

gg_bar_plot_opt( Pla, Gga ) :-
	( \+compound(Pla); is_list(Pla); Pla = (+ _); compound(Pla,c,_) ),
	!,
	Gga = Pla.
gg_bar_plot_opt( Pla, Gga ) :-
	compound( Pla, Name, Args ),
	maplist( gg_bar_plot_gg_arg, Args, GGArgs ),
	compound( Gga, Name, GGArgs ).

gg_bar_plot_gg_arg( Pla, Gga ) :-
	( \+compound(Pla); is_list(Pla); Pla = (+ _); compound(Pla,c,_) ),
	!,
	Gga = Pla.
gg_bar_plot_gg_arg( Pla, Gga ) :-
	compound( Pla, Name, [Arg] ),
	gg_bar_plot_opt( Arg, Sub ),
	Gga = ( Name = Sub ).

pl_terms_gg_pairs( Arg, Pair ) :-
	compound( Arg, Name, [SubArg] ),
	( \+ compound( SubArg ) ; is_list( SubArg ) ),
	!,
	Pair = (Name = SubArg).
pl_terms_gg_pairs( Arg, Pair ) :-
	compound( Arg, Name, Args ),
	maplist( pl_terms_gg_pairs, Args, Pairs ),
	Pair = (Name=Pairs). %
