:- ensure_loaded( library(terms) ).
% note: swi doesnt have terms lib. implement (at least variant).

/** list_frequency( +List, -Frequencies ).
    list_frequency( +List, +Expected, -Frequencies ).

FrequenciesOfList is a list of Terms-Freq -pairs
of how many times each term (and its variants) appear in the List.

Expected lists all expected elements in list, Frequencies will then
contain one counter per Expected value, including any zeros. In this version, 
the elements are not included, sl 
In list_frequency/2 version, there are no zeros counts and the order is in 

==
?- list_frequency( [a,b,a,b,c], Freqs ).
Freqs = [a-2, b-2, c-1].

?- list_frequency( [a,b,a,b,c], [d,c,b,a], Freqs ).
Freqs = [d-0, c-1, b-2, a-2].
==

@author nicos angelopoulos
@version  0.2 2015/11/25, added /3 version and examples

*/

list_frequency( [H|T], [H-HTimes|CountedT] ) :-
        list_frequency_1( T, H, 1, HTimes, ReducedT ),
        list_frequency( ReducedT, CountedT ).
list_frequency( [], [] ).

list_frequency_1( [H|T], El, Acc, Count, RedT ) :-
        variant( El, H ),
        !,
        Acc1 is Acc + 1,
        list_frequency_1( T, El, Acc1, Count, RedT ).
list_frequency_1( [H|T], El, Acc, Count, [H|RedT] ) :-
        list_frequency_1( T, El, Acc, Count, RedT ).
list_frequency_1( [], _El, Acc, Acc, [] ).

list_frequency( List, Expct, Freqs ) :-
	findall( Elem-0, member(Elem,Expct), Counters ),
	list_frequency_expected( List, Counters, Freqs ).

list_frequency_expected( [], Freqs, Freqs ).
list_frequency_expected( [H|T], Counters, Freqs ) :-
	list_frequency_expected_increase_counter( Counters, H, Next ),
	list_frequency_expected( T, Next, Freqs ).

list_frequency_expected_increase_counter( [], Elem, _Next ) :-
	throw( no_counter_for(Elem) ).
list_frequency_expected_increase_counter( [Elem-Curr|T], Elem, [Elem-Next|T] ) :-
	!,
	Next is Curr + 1.
list_frequency_expected_increase_counter( [H|T], Elem, [H|Next] ) :-
	list_frequency_expected_increase_counter( T, Elem, Next ).
